/************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER
 *
 * Copyright 2024 J David Eisenberg. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0. You can also
 * obtain a copy of the License at http://odftoolkit.org/docs/license.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************************/

import org.w3c.dom.Node;

import org.odftoolkit.odfdom.doc.OdfPresentationDocument;
import org.odftoolkit.odfdom.doc.presentation.OdfSlide.SlideLayout;

import org.odftoolkit.odfdom.dom.OdfContentDom;
import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeAutomaticStyles;

import org.odftoolkit.odfdom.dom.style.OdfStyleFamily;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Scanner;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import javax.imageio.ImageIO;
import java.net.URI;

import org.odftoolkit.odfdom.dom.element.draw.DrawFrameElement;
import org.odftoolkit.odfdom.dom.element.draw.DrawGElement;
import org.odftoolkit.odfdom.dom.element.draw.DrawImageElement;
import org.odftoolkit.odfdom.incubator.doc.draw.OdfDrawImage;
import org.odftoolkit.odfdom.dom.element.draw.DrawPageElement;
import org.odftoolkit.odfdom.dom.element.draw.DrawRectElement;
import org.odftoolkit.odfdom.dom.element.draw.DrawTextBoxElement;

import org.odftoolkit.odfdom.dom.element.office.OfficePresentationElement;

import org.odftoolkit.odfdom.incubator.doc.style.OdfStyle;
import org.odftoolkit.odfdom.dom.element.style.StyleGraphicPropertiesElement;
import org.odftoolkit.odfdom.dom.element.style.StyleTextPropertiesElement;
import org.odftoolkit.odfdom.incubator.doc.text.OdfTextParagraph;
import org.odftoolkit.odfdom.dom.element.text.TextPElement;
import org.odftoolkit.odfdom.dom.attribute.presentation.PresentationClassAttribute;

/**
 * Create an ODP file.
 * This program will create an OpenDocument Presentation from
 * data contained in a text file.
 *
 * Each slide shows the image of a building or monument.
 * The slide's title gives the name of the building.
 * The slide also contains a two-by-two "table" giving the city and
 * country where the building is found.
 * 
 * @author J David Eisenberg
 */
public class TravelPresentation {

    /* Variables for the ODF file */
    OdfPresentationDocument outputDocument;
    OfficePresentationElement officePresentation;
    OdfContentDom contentDom;
    OdfOfficeAutomaticStyles contentAutoStyles;

    String slideTitleStyleName;
    String imageStyleName;
    String fauxTableRectangleStyleName;
    String fauxTableTextStyleName;

    void run(String[] args) {
        setupOutputDocument();
        outputDocument.deleteSlideByIndex(0);
        addAutomaticStyles();
        createPages(args[0]);
        saveOutputDocument(args[1]);
    }

    private void setupOutputDocument() {
        try {
            outputDocument =
                OdfPresentationDocument.newPresentationDocument();
            officePresentation = outputDocument.getContentRoot();

            contentDom = outputDocument.getContentDom();
            contentAutoStyles = contentDom.getOrCreateAutomaticStyles();
        }
        catch (Exception e) {
            System.err.println("Unable to create output file.");
            System.err.println(e.getMessage());
            outputDocument = null;
        }
    }


    void addAutomaticStyles() {
        OdfStyle style;

        /* The style for the slide title */
        style = contentAutoStyles.newStyle(OdfStyleFamily.Presentation);
        slideTitleStyleName = style.getStyleNameAttribute();
        style.setProperty(StyleGraphicPropertiesElement.AutoGrowHeight, "true");
        style.setProperty(StyleGraphicPropertiesElement.MinHeight, "3cm");
        style.setProperty(StyleTextPropertiesElement.FontFamily, "Helvetica");
        style.setProperty(StyleTextPropertiesElement.FontSize, "44pt");

        /* Style for the image element */
        style = contentAutoStyles.newStyle(OdfStyleFamily.Graphic);
        imageStyleName = style.getStyleNameAttribute();
        style.setProperty(StyleGraphicPropertiesElement.MinHeight, "10cm");
        style.setProperty(StyleGraphicPropertiesElement.Clip,
            "rect(0cm, 0cm, 0cm, 0cm)");

        /* Graphic style for the table rectangles */
        style = contentAutoStyles.newStyle(OdfStyleFamily.Graphic);
        fauxTableRectangleStyleName = style.getStyleNameAttribute();
        style.setProperty(StyleGraphicPropertiesElement.FillColor,
            "#ffffff");
        style.setProperty(StyleGraphicPropertiesElement.TextareaVerticalAlign,
            "middle");
        style.setProperty(StyleGraphicPropertiesElement.PaddingLeft, "8pt");

        /* Style for the text in the table */
        style = contentAutoStyles.newStyle(OdfStyleFamily.Paragraph);
        fauxTableTextStyleName = style.getStyleNameAttribute();
        style.setProperty(StyleTextPropertiesElement.FontFamily, "Helvetica");
        style.setProperty(StyleTextPropertiesElement.FontSize, "16pt");
    }

    void createPages(String inputFileName) {

        try (Scanner slideInfoScanner = new Scanner(new File(inputFileName))) {
            int counter = 1;
            while (slideInfoScanner.hasNextLine()) {
                String info = slideInfoScanner.nextLine();
                if (!info.startsWith("//")) {
                    String[] parts = info.split(",");
                    if (parts.length == 4) {
                        createOnePage(parts, counter);
                    } else {
                        System.err.println("\"" + info + "\" does not have the correct number of items.");
                    }
                    counter++;
                }
            }

        } catch (Exception e) {
                // do nothing; no pictures will be added.
        }
    }

    /*
     * Info contains title, country, city, image path
     */
    void createOnePage(String[] info, int counter) {

        DrawPageElement page = officePresentation.newDrawPageElement(null);
        page.setDrawNameAttribute("pg" + Integer.toString(counter));

        /* Create the title in a <draw:frame> element */
        DrawFrameElement frame = page.newDrawFrameElement();
        frame.setPresentationStyleNameAttribute(slideTitleStyleName);
        frame.setPresentationClassAttribute(
            PresentationClassAttribute.Value.TITLE.toString());
        frame.setSvgXAttribute("1.4cm");
        frame.setSvgYAttribute("0.85cm");
        frame.setSvgWidthAttribute("25cm");
        frame.setSvgHeightAttribute("3.5cm");

        /*
            * Which contains a <draw:text-box> that has a
            * <text:p> inside it
            */
        DrawTextBoxElement textBox = frame.newDrawTextBoxElement();
        TextPElement para = textBox.newTextPElement();
        para.setTextContent(info[0]);

        /*
            * Now put in the image. This is all included in a
            * try/catch block; if there's some problem accessing
            * the image, the frame just doesn't get added to the
            * page.
            */
        try {
            frame = page.newDrawFrameElement();
            frame.setPresentationStyleNameAttribute(imageStyleName);
            frame.setPresentationClassAttribute(
                PresentationClassAttribute.Value.GRAPHIC.toString());
            frame.setSvgXAttribute("14cm");
            frame.setSvgYAttribute("6cm");

            OdfDrawImage image = new OdfDrawImage(contentDom);
            String imgPath = image.newImage(new URI(info[3]));
            image.setXlinkHrefAttribute(imgPath);
            frame.appendChild(image);


            /*
             * Always make the picture 12cm wide,
             * and adjust the height in proportion to the width
             */
            BufferedImage buffer = ImageIO.read(new File(info[3]));
            frame.setSvgWidthAttribute("12cm");
            frame.setSvgHeightAttribute(Double.toString(
                (12.0 * buffer.getHeight()) / buffer.getWidth())
                    + "cm");


            page.appendChild(createDrawnTable(info));
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /*
     * Create the table by drawing four rectangles with text in them
     */
    private DrawGElement createDrawnTable(String[] info) {
        DrawGElement group = new DrawGElement(contentDom);

        group.appendChild(createTextRectangle(
            "Country", "1.5cm", "6cm", "4.7cm", "1cm"));
        group.appendChild(createTextRectangle(
            info[1], "6.2cm", "6cm", "7cm", "1cm"));
        group.appendChild(createTextRectangle(
            "City", "1.5cm", "7cm", "4.7cm", "1cm"));
        group.appendChild(createTextRectangle(
            info[2], "6.2cm", "7cm", "7cm", "1cm"));
        return group;
    }

    /*
     * Create a rectangle that contains text.
     * @param textValue the text to display inside the rectangle
     * @param x coordinate of left corner of rectangle
     * @param y coordinate of upper corner of rectangle
     * @param width width of rectangle
     * @param height height of rectangle
     */
    private DrawRectElement createTextRectangle(String textValue, String x, String y,
        String width, String height) {
        DrawRectElement r = new DrawRectElement(contentDom);
        OdfTextParagraph p;

        r.setSvgXAttribute(x);
        r.setSvgYAttribute(y);
        r.setSvgWidthAttribute(width);
        r.setSvgHeightAttribute(height);
        r.setDrawStyleNameAttribute(fauxTableRectangleStyleName);
        
        p = new OdfTextParagraph(contentDom);
        p.setStyleName(fauxTableTextStyleName);
        p.appendChild(contentDom.createTextNode(textValue));
        r.appendChild(p);
        return r;
    }

    void saveOutputDocument(String outputFileName)
    {
        try
        {
            outputDocument.save(outputFileName);
        }
        catch (Exception e)
        {
            System.err.println("Unable to save document.");
            System.err.println(e.getMessage());
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        TravelPresentation app = new TravelPresentation();
        if (args.length == 2) {
            app.run(args);
        } else {
            System.err.println("Usage: TravelPresentation infile outfile");
        }
    }
}
